from pyndri.dictionary import Dictionary, extract_dictionary

from pyndri_ext import Index as __IndexBase
from pyndri_ext import QueryEnvironment, krovetz_stem, porter_stem, tokenize

import os

__all__ = [
    'Index',
    'Dictionary',
    'QueryEnvironment',
    'extract_dictionary',
    'krovetz_stem',
    'porter_stem',
    'tokenize',
    'escape',
]


def escape(input):
    return input.translate({
        ord('('): None,
        ord(')'): None,
        ord('\''): None,
        ord('\"'): None,
        ord('.'): ' ',
        ord(':'): ' ',
        ord('\t'): ' ',
        ord('/'): ' ',
        ord('&'): ' ',
        ord(','): ' ',
        ord('-'): ' ',
        ord('?'): ' ',
        ord('+'): ' ',
        ord(';'): ' ',
        ord('`'): None,
        ord('$'): None,
        ord('<'): ' ',
        ord('>'): ' ',
        ord('%'): ' ',
        ord('@'): ' ',
        ord('\\'): ' ',
        ord('*'): ' ',
    })


class Index(__IndexBase):

    def __init__(self, *args, **kwargs):
        super(Index, self).__init__(*args, **kwargs)

        self.__default_query_env = QueryEnvironment(self)

    def query(self, *args, **kwargs):
        assert self.__default_query_env is not None, \
            'Index has been closed.'

        return self.__default_query_env.query(*args, **kwargs)

    def tokenize(self, string):
        return [
            self.process_term(token)
            for token in
            tokenize(escape(string))]

    def __len__(self):
        return self.maximum_document() - self.document_base()

    def __repr__(self):
        return '<pyndri.Index of {} documents>'.format(len(self))

    def close(self):
        assert self.__default_query_env is not None, \
            'Index has been closed.'

        del self.__default_query_env
        self.__default_query_env = None


class __IndexOpener(object):

    def __init__(self, path):
        if not os.path.isdir(path):
            raise IOError('Index path is not a directory.')
        elif not os.path.exists(os.path.join(path, 'manifest')):
            raise IOError('Index manifest not found.')

        self.path = path

    def __enter__(self):
        self.index = Index(self.path)
        return self.index

    def __exit__(self, type, value, traceback):
        self.index.close()

        del self.index
        self.index = None


open = __IndexOpener


class TFIDFQueryEnvironment(QueryEnvironment):

    def __init__(self, index, k1=1.2, b=0.75):
        super(TFIDFQueryEnvironment, self).__init__(
            index, baseline='tfidf,k1:{k1:.5f},b:{b:.5f}'.format(
                k1=k1, b=b))


class OkapiQueryEnvironment(QueryEnvironment):

    def __init__(self, index, k1=1.2, b=0.75, k3=7.0):
        super(OkapiQueryEnvironment, self).__init__(
            index, baseline='okapi,k1:{k1:.5f},b:{b:.5f},k3:{k3:.5f}'.format(
                k1=k1, b=b, k3=k3))



import pyndri
import sys

if len(sys.argv) <= 1:
    print('Usage: python {0} <path-to-indri-index>'.format(sys.argv[0]))

    sys.exit(0)

with pyndri.open(sys.argv[1]) as index:
    token2id, id2token, id2df = index.get_dictionary()
    print(token2id)
    print(id2token)

    id2tf = index.get_term_frequencies()

    print(len(token2id), len(id2tf))

    print('Index contains %d documents.' % index.document_count())



import pyndri

print('predictions',
      pyndri.krovetz_stem('predictions'),
      pyndri.porter_stem('predictions'))  # prediction, predict
print('marketing',
      pyndri.krovetz_stem('marketing'),
      pyndri.porter_stem('marketing'))  # marketing, market
print('strategies',
      pyndri.krovetz_stem('strategies'),
      pyndri.porter_stem('strategies'))  # strategy, strategi



import pyndri
import sys

if len(sys.argv) <= 2:
    print('Usage: python {0} <path-to-indri-index> '
          '[<index-name>] [<max-document-length>]'.format(sys.argv[0]))

    sys.exit(0)

with pyndri.open(sys.argv[1]) as index:
    for document_id in range(index.document_base(), index.maximum_document()):
        if index.document_length(document_id) < int(sys.argv[2]):
            print(index.document(document_id)[0])



import gensim
import pyndri


class IndriSentences(gensim.interfaces.CorpusABC):
    """Integrates an Index with Gensim's word2vec implementation."""

    def __init__(self, index, dictionary, max_documents=None):
        assert isinstance(index, pyndri.Index)

        self.index = index
        self.dictionary = dictionary

        self.max_documents = max_documents

    def _maximum_document(self):
        if self.max_documents is None:
            return self.index.maximum_document()
        else:
            return min(
                self.max_documents + self.index.document_base(),
                self.index.maximum_document())

    def __iter__(self):
        for int_doc_id in range(self.index.document_base(),
                                self._maximum_document()):
            ext_doc_id, tokens = self.index.document(int_doc_id)

            yield tuple(
                self.dictionary[token_id]
                for token_id in tokens
                if token_id > 0 and token_id in self.dictionary)

    def __len__(self):
        return self._maximum_document() - self.index.document_base()



import pyndri
import sys

import numpy as np
import scipy.stats.mstats

if len(sys.argv) <= 1:
    print('Usage: python {0} <path-to-indri-index> [<index-name>]'.format(
        sys.argv[0]))

    sys.exit(0)

with pyndri.open(sys.argv[1]) as index:
    num_documents = 0
    mean = 0.0
    M2 = 0.0

    min_ = np.inf
    max_ = -np.inf

    lengths = []

    for document_id in range(index.document_base(), index.maximum_document()):
        x = float(index.document_length(document_id))
        lengths.append(x)

        num_documents += 1
        delta = x - mean
        mean += delta / num_documents
        M2 += delta * (x - mean)

        min_ = min(min_, x)
        max_ = max(max_, x)

    std = np.sqrt(M2 / (num_documents - 1))

    mode = scipy.stats.mstats.mode(lengths)
    median = np.median(lengths)

    prefix = '' if len(sys.argv) == 2 else '{}_'.format(sys.argv[2].upper())

    print('export {}NUM={}'.format(prefix, num_documents))
    print('export {}LENGTH_MEDIAN={}'.format(prefix, median))
    print('export {}LENGTH_MODE={}'.format(prefix, mode.mode[0]))
    print('export {}LENGTH_MEAN={}'.format(prefix, mean))
    print('export {}LENGTH_MIN={}'.format(prefix, min_))
    print('export {}LENGTH_MAX={}'.format(prefix, max_))
    print('export {}LENGTH_STD={}'.format(prefix, std))
    print('export {}TOTAL_TERMS={}'.format(prefix, index.total_terms()))
    print('export {}UNIQUE_TERMS={}'.format(prefix, index.unique_terms()))



import pyndri
import sys

if len(sys.argv) <= 1:
    print('Usage: python {0} <path-to-indri-index>'.format(sys.argv[0]))

    sys.exit(0)

with pyndri.open(sys.argv[1]) as index:
    for document_id in range(index.document_base(), index.maximum_document()):
        # Prints pairs of form (external_document_id, terms).
        #
        # Example:
        #   ('eUK950521', (877, 2171, 797, 877, 2171, 2771, 1768, 1262, 2171))
        #   ('eUK436208', (381, 3346))
        print(index.document(document_id))

    # The following line will raise an exception, as there is no document
    # with internal identifier 0.
    print(index.document(0))



import operator
import pyndri
import sys

if len(sys.argv) <= 1:
    print('Usage: python {0} <path-to-indri-index>'.format(sys.argv[0]))

    sys.exit(0)

with pyndri.open(sys.argv[1]) as index:
    results = index.query('hello world', results_requested=10)

    for int_document_id, score in results:
        ext_document_id, _ = index.document(int_document_id)

        print('Document {ext_document_id} retrieved with score {score}.'.format(
            ext_document_id=ext_document_id, score=score))

    print()

    results = index.query(
        'hello world',
        document_set=map(
            operator.itemgetter(1),
            index.document_ids(['eUK306804', 'eUK700967'])),
        results_requested=-5,
        include_snippets=True)

    for int_document_id, score, snippet in results:
        ext_document_id, _ = index.document(int_document_id)

        print('Document {ext_document_id} ("{snippet}") '
              'retrieved with score {score}.'.format(
                  ext_document_id=ext_document_id,
                  snippet=snippet.replace('\n', ' '),
                  score=score))



import collections
import heapq
import pyndri
import logging

__all__ = [
    'Dictionary',
    'extract_dictionary',
]


class Dictionary(object):
    """
    Interface to Indri dictionary.

    This class is compatible with gensim.
    """

    def __init__(self, token2id, id2token, id2df, **kwargs):
        assert len(token2id) == len(id2token)

        self.token2id = token2id
        self.id2token = id2token

        self.dfs = id2df

        if 'krovetz_stemming' in kwargs:
            raise NotImplementedError('krovetz_stemming was deprecated '
                                      'in favour of Index.process_term.')

    def __getitem__(self, token_id):
        return self.id2token[token_id]

    def __contains__(self, token_id):
        return token_id in self.id2token

    def __iter__(self):
        return self.iterkeys()

    def keys(self):
        return self.iterkeys()

    def iterkeys(self):
        return self.id2token.keys()

    def __len__(self):
        return len(self.id2token)

    def __str__(self):
        return 'IndriDictionary({0} unique tokens)'.format(
            len(self.id2token))

    def translate_token(self, token):
        return self.token2id.get(token, None)

    def has_token(self, token):
        return token in self.token2id

    def doc2bow(self, document):
        if isinstance(document, str) or isinstance(document, bytes):
            raise TypeError(
                'doc2bow expects an iterable of unicode tokens on input, '
                'not a single string')

        counter = collections.defaultdict(int)

        for token in document:
            if isinstance(token, int):
                token_id = token
            else:
                assert isinstance(token, str)

                token_id = self.translate_token(token)

            if token_id is None:
                continue

            counter[token_id] += 1

        return sorted(counter.items())


def extract_dictionary(index,
                       max_terms=None, make_contiguous=False,
                       **kwargs):
    assert isinstance(index, pyndri.Index)

    logging.debug('Extracting dictionary from index %s.', index)
    token2id, id2token, id2df = index.get_dictionary()

    if max_terms is not None and max_terms:
        id2tf = index.get_term_frequencies()

        top_terms_ids = set(heapq.nlargest(
            max_terms,
            iterable=id2tf, key=lambda id: id2tf[id]))

        token2id = {key: value for key, value in token2id.items()
                    if value in top_terms_ids}
        id2token = {key: value for key, value in id2token.items()
                    if key in top_terms_ids}
        id2df = {key: value for key, value in id2df.items()
                 if key in top_terms_ids}

    if make_contiguous:
        # Dictionary does not have contiguous identifiers; wrap it.
        indri_id2cid = {id: cid for cid, id in enumerate(id2token)}
        id2token = {
            cid: id2token[id] for id, cid in indri_id2cid.items()}
        token2id = {
            token: cid for cid, token in id2token.items()}
        id2df = {
            cid: id2df[indri_id] for indri_id, cid in indri_id2cid.items()}
    else:
        indri_id2cid = None

    dictionary = Dictionary(token2id, id2token, id2df, **kwargs)

    if make_contiguous:
        return dictionary, indri_id2cid
    else:
        return dictionary



import os
from distutils.core import setup, Extension

pyndri_ext = Extension(
    'pyndri_ext',
    sources=['src/pyndri.cpp'],
    libraries=['indri', 'z', 'pthread', 'm'],
    library_dirs=list(
        filter(len, os.environ.get('LD_LIBRARY_PATH', '').split(':'))),
    define_macros=[('_GLIBCXX_USE_CXX11_ABI', '0'),
                   ('P_NEEDS_GNU_CXX_NAMESPACE', '1'),
                   ('_FILE_OFFSET_BITS', '64')],
    undef_macros=['NDEBUG'])

setup(name='pyndri',
      version='0.2',
      description='pyndri is a Python interface to the Indri search engine',
      author='Christophe Van Gysel',
      author_email='cvangysel@uva.nl',
      ext_modules=[pyndri_ext],
      packages=['pyndri'],
      package_dir={'pyndri': 'py'},
      python_requires='>=3',
      url='https://github.com/cvangysel/pyndri',
      download_url='https://github.com/cvangysel/pyndri/tarball/0.2',
      keywords=['indri', 'language models', 'retrieval', 'indexing'],
      classifiers=[
          'Development Status :: 3 - Alpha',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python',
          'Programming Language :: C++',
          'Intended Audience :: Science/Research',
          'Operating System :: POSIX :: Linux',
          'Topic :: Text Processing :: Indexing',
      ])



"""
Query environments example.

This example shows how to use query environments with different
retrieval models. The query() method supports the same arguments
as Index.query(); see query.py.
"""

import pyndri
import sys

if len(sys.argv) <= 1:
    print('Usage: python {0} <path-to-indri-index>'.format(sys.argv[0]))

    sys.exit(0)

with pyndri.open(sys.argv[1]) as index:
    # Constructs a QueryEnvironment that uses a
    # language model with Dirichlet smoothing.
    lm_query_env = pyndri.QueryEnvironment(
        index, rules=('method:dirichlet,mu:5000',))
    print(lm_query_env.query('hello world'))

    # Constructs a QueryEnvironment that uses the TF-IDF retrieval model.
    #
    # See "Baseline (non-LM) retrieval"
    # (https://lemurproject.org/doxygen/lemur/html/IndriRunQuery.html)
    tfidf_query_env = pyndri.TFIDFQueryEnvironment(index)
    print(tfidf_query_env.query('hello world'))

    # Constructs a QueryEnvironment that uses the Okapi BM25 retrieval model.
    #
    # See "Baseline (non-LM) retrieval"
    # (https://lemurproject.org/doxygen/lemur/html/IndriRunQuery.html)
    bm25_query_env = pyndri.OkapiQueryEnvironment(index)
    print(bm25_query_env.query('hello world'))



import copy
import gensim
import logging
import pyndri
import pyndri.compat
import sys

logging.basicConfig(level=logging.INFO)

if len(sys.argv) <= 1:
    logging.error('Usage: python %s <path-to-indri-index>'.format(sys.argv[0]))

    sys.exit(0)

logging.info('Initializing word2vec.')

word2vec_init = gensim.models.Word2Vec(
    size=300,  # Embedding size
    window=5,  # One-sided window size
    sg=True,  # Skip-gram.
    min_count=5,  # Minimum word frequency.
    sample=1e-3,  # Sub-sample threshold.
    hs=False,  # Hierarchical softmax.
    negative=10,  # Number of negative examples.
    iter=1,  # Number of iterations.
    workers=8,  # Number of workers.
)

with pyndri.open(sys.argv[1]) as index:
    logging.info('Loading vocabulary.')
    dictionary = pyndri.extract_dictionary(index)
    sentences = pyndri.compat.IndriSentences(index, dictionary)

    logging.info('Constructing word2vec vocabulary.')

    # Build vocab.
    word2vec_init.build_vocab(sentences, trim_rule=None)

    models = [word2vec_init]

    for epoch in range(1, 5 + 1):
        logging.info('Epoch %d', epoch)

        model = copy.deepcopy(models[-1])
        model.train(sentences)

        models.append(model)

    logging.info('Trained models: %s', models)
